﻿namespace UltraLabelHotkey
{
    using System.Drawing;
    using System.Windows.Forms;
    using Infragistics.Win;
    using static Infragistics.Win.ImageAndTextUIElement;

    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            new Infragistics.Win.UIElementViewer.UIElementViewerForm().Show();
            this.ultraLabel1.DrawFilter = new DF(Color.Red);
            this.ultraLabel2.DrawFilter = new DF(Color.Blue);
            this.ultraLabel3.DrawFilter = new DF(Color.Green);
            this.ultraLabel4.DrawFilter = new DF(Color.Magenta);
        }
    }

    internal class DF : IUIElementDrawFilter
    {
        private Color color;

        public DF(Color color)
        {
            this.color = color;
        }

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            var textElement = drawParams.Element as ImageAndTextDependentTextUIElement;
            if(textElement != null)
            {
                //  Get the text of the label
                var text = textElement.Text;

                //  Get the index of the star symbol
                var starIndex = text.IndexOf('*');

                //  Get the index of ampersand symbol
                var focusCharIndex = text.IndexOf('&');

                //  If there is ampersand in the text remove it. Also if the star is after the
                //  ampersand decrease the position of the star as we are making the length of
                //  the text before the star one letter shorter
                if(focusCharIndex > -1)
                {
                    text = text.Substring(0, focusCharIndex) + text.Substring(focusCharIndex + 1);
                    if(starIndex > focusCharIndex)
                    {
                        starIndex--;
                    }
                }

                //  Get the location of the start of the text, the font and the color of the text
                var location = textElement.RectInsideBorders.Location;
                var font = textElement.Control.Font;
                var brush = drawParams.TextBrush;
                var originalColor = ((SolidBrush)drawParams.TextBrush).Color;

                var index = 0;

                //  Draw each letter as necessary by yourself via drawParams' Graphic object
                foreach(var letter in text)
                {
                    //  If we are at the star change the color of the brush to the one you need
                    if(index == starIndex)
                    {
                        ((SolidBrush)brush).Color = this.color;
                    }

                    //  If we are at focus char make font Underline
                    if(index == focusCharIndex)
                    {
                        font = new Font(font, FontStyle.Underline);
                    }

                    //  Draw current char
                    drawParams.Graphics.DrawString(letter.ToString(), font, brush, location);

                    //  Measure the width of the current char and offset the location accordingly
                    //  to be able to draw the next char at the correct position
                    var letterWidth = drawParams.Graphics.MeasureString(letter.ToString(), font, int.MaxValue, StringFormat.GenericTypographic).Width;
                    location.Offset((int)letterWidth, 0);

                    //  Reset the brush and font
                    ((SolidBrush)brush).Color = originalColor;
                    font = textElement.Control.Font;

                    index++;
                }

                //  Return true so no other painting will be performed
                return true;
            }

            return false;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            //  If the element to be drawn is ImageAndTextDependentTextUIElement return just before
            //  the ForeGround, text, is drawn
            if(drawParams.Element is ImageAndTextDependentTextUIElement)
            {
                return DrawPhase.BeforeDrawForeground;
            }
            return DrawPhase.None;
        }
    }
}
